/**
 * Concat multiple files into one.
 * @memberof apeman-commons-compiling
 * @function concatFiles
 * @param {string[]} src - Filenames to concat
 * @param {string} dest - Destination file path.
 * @param {object} [options] - Optional settings.
 * @param {boolean} [options.unique] - SKip duplicate source.
 * @param {boolean} [options.log] - Should emit log.
 * @param {function} [callback] - Callback when done.
 */

"use strict";

var async = require('async'),
    fs = require('fs'),
    path = require('path'),
    colorprint = require('colorprint'),
    writeout = require('writeout'),
    fileconcat = require('fileconcat'),
    argx = require('argx');

/** @lends concatFiles */
function concatFiles(src, dest, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    dest = args.pop('string');
    src = args.remain().reduce(function (a, b) {
        return a.concat(b);
    }, []);

    if (!dest) {
        callback(new Error('dest is required.'));
        return;
    }


    var tmp = path.resolve(path.dirname(dest), '.' + path.basename(dest) + '.tmp.' + new Date().getTime());
    async.series([
        function (callback) {
            fileconcat(src, tmp, {
                unique: options.unique,
                mkdirp: true
            }, callback);
        },
        function (callback) {
            async.waterfall([
                function (callback) {
                    fs.readFile(tmp, callback);
                },
                function (content, callback) {
                    writeout(dest, content, {
                        mkdirp: true,
                        skipIfIdentical: true
                    }, callback);
                },
                function (result, callback) {
                    var shouldLog = !result.skipped && options.log;
                    if (shouldLog) {
                        colorprint.debug('File generated:', dest);
                    }
                    callback(null, result);
                }
            ], callback);
        }
    ], function (err, results) {
        fs.unlink(tmp, function () {
            callback(err, results.pop());
        });
    });

}

module.exports = concatFiles;